@@ -46,8 +46,18 @@ ReadHosts(char *HostFile)
 
     {
 
-      /*WGK make sure HostFile is owned by us and has restricted write permissions.*/
+
       struct stat sbuf;
+
+      /*First make sure that compilation properly defined HostFile.*/
+
+      if(!HostFile || !*HostFile) {
+	fprintf(stderr, "ReadHosts: Received null HostFile. Make sure QHOSTSFILE has been properly set in config.h.\nNormally, this is automatically done by `configure'.\nIf this was not done by `./configure' on your platform, this is a bug and you should report it to bug-queue@gnu.org\n\n");
+	fprintf(stderr,"ReadHosts: Invalid HostFile. Exiting.\n");
+	exit(2);
+      }
+
+      /*WGK make sure HostFile is owned by us and has restricted write permissions.*/
       stat(HostFile, &sbuf);
 
 #ifdef NO_ROOT
@@ -141,8 +151,10 @@ ReadHosts(char *HostFile)
 
       if((Hosts[i].host = strdup(canonicalhost(&TmpStr[j]))) == NULL)
       {
-          printf("ERROR allocating memory for hosts struct\n");
-          exit(2);
+	/*WGK 1999/01/25 Bugfix. This is probably a bogus host.*/
+          fprintf(stderr,"WARNING: Unable to resolve %s or out of memory\n", TmpStr[j]);
+          NHosts--;
+	  continue;
         }
       i++;
       
@@ -163,11 +175,34 @@ for(i=lhost;*i!='\0';++i) if (*i=='.') break;
 return(lhost);
 }
 
+int countdots(host)
+char *host;
+{
+int count = 0;
+char *ptr = host;
+while((ptr!=NULL)&&((ptr=index(ptr,'.'))!=NULL)) {
+  count++;
+  ptr++;
+}
+return(count);
+}
+
 char *canonicalhost(host)
 char *host;
 {
+char *ret;
+char *best;
+int maxdots = 0;
+int i;
+int tmp;
+static char cache[MAXHOSTNAMELEN + 1];
+
 /* WGK 1995
 
+Bugs fixed and changed to return the first name with most dots 1999/02/05.
+
+More bugs fixed. Added static cache for stuff WGK 1999/02/13 .
+
 
 This will save us problems when people upgrade their gethostname()
 from host to host.domainname during an OS upgrade. It has the additional
@@ -175,14 +210,43 @@ advantage that it correctly distinguishes between otherwise identically
 named hosts on different domains, in case anyone ever wants to try to
 exploit that possible security hole,
 and resolves aliases to a single official name for each host.*/
+
+/*Move host string to our own static area in case host comes from static
+DNS libc areas. This code just lets us call canonicalhost on gethosbyname()
+stuff safely.*/
   struct hostent *thathost;
-  if((thathost = gethostbyname(host))==NULL) {
-    fprintf(stderr,"gethostbyname on %s returned error.\n",host);
-    return(host);
+
+strncpy(cache,host,MAXHOSTNAMELEN);
+cache[MAXHOSTNAMELEN-1] = 0;
+
+/*Appending the domainname from getdomainname() as someone suggested
+ will not work because 
+getdomainname is the NIS domain, not the BIND domain, and they are quite
+different in many installations! I've reluctantly given up and now
+simply return an unqualified domainname if we have a lazy gethostbyname().*/
+
+
+  if((thathost = gethostbyname(cache))==NULL) {
+    fprintf(stderr,"gethostbyname on %s returned error.\n",cache);
+        return(cache);
   }
-  return(thathost->h_aliases[0]);
+
+best = NULL;
+
+#define find_maxdots(a,b) if ((tmp=countdots(a)) > b) { b = tmp; best = a; }
+
+find_maxdots(cache,maxdots);
+find_maxdots(thathost->h_name,maxdots);
+for(i = 0;thathost->h_aliases[i]!=NULL;i++) find_maxdots(thathost->h_aliases[i],maxdots);
+
+/*Convert host to lowercase.*/
+for(ret=best;*ret!=0;ret++) *ret=tolower(*ret);
+
+return(best);
 }
 
+
+
 char * mymalloc(n)
 {
 	register char *p;
