@@ -245,7 +245,7 @@ case 'l': {
 }
  case 'h': {
    if (!strcmp(optarg,"+")) break; /* '+' is wildcard; ignore.*/
-   if (strlen(optarg) > 1000) {
+   if (strlen(optarg) > MAXHOSTNAMELEN) {
      fprintf(stderr, "Queue: hostname too long.\n");
      exit(1);
    }
@@ -256,7 +256,7 @@ case 'l': {
    break;}
  case 'H': {
    if (!strcmp(optarg,"+")) break; /* '+' is wildcard; ignore.*/
-   if (strlen(optarg) > 1000) {
+   if (strlen(optarg) > MAXHOSTNAMELEN) {
      fprintf(stderr, "Queue: hostname too long.\n");
      exit(1);
    }
@@ -327,6 +327,7 @@ command line. This is then followed by the command line;
 no '--' is used with qsh.*/
 
 if(strcmp(invoke,"qsh")==0) {
+   if (optind >= argc) {Qshusage(); exit(1);}
    if (!strcmp(argv[optind],"+")) optind++;
 else{
      if (strlen(argv[optind]) > 1000) {
@@ -340,6 +341,8 @@ else{
 }
 }
 
+if (optind >= argc) {Usage(); exit(1);}
+
  if(!Hosts) {
 if(ReadHosts(QHOSTSFILE) < 0)
   {
@@ -989,10 +992,12 @@ in the style of TCP/IP wrapper.	 */
 	      exit(1);
 	    }
 	  }
-		
-	  hostname = hostp->h_aliases[0]; /*fully qualified*/
+	  /*Can't call canonicalhost with static hostp->h_name*/
+	  strncpy(remotehost, hostp->h_name, sizeof(remotehost) - 1);
+	  remotehost[sizeof(remotehost) - 1] = 0;
+	  hostname = canonicalhost(remotehost);
 	} else
-	hostname = inet_ntoa(address1.sin_addr); /*Our DNS must be down and this will probably fail.*/
+	hostname = "unresolved"; /*Our DNS must be down and this will probably fail.*/
 
 if(!validhost(hostname,socket)) exit(1);
 
@@ -1410,12 +1415,19 @@ syserror(s)
 		syslog(LOG_ALERT, "queue: %s", s);
 }
 
-int validhost(hostname, socket)
-char *hostname;
+int validhost(hostcpy, socket)
+char *hostcpy;
 int socket;
 {
+char buf[MAXHOSTNAMELEN + 1];
 register int i;
 int flag = 1;
+char *hostname = buf;
+
+hostname = strncpy(hostname,hostcpy,MAXHOSTNAMELEN); /*Unfortunately, the static area from
+gethostbynames is completely unstable on some platforms and will
+not survive a call to stat(), which is used in readhosts.*/
+hostname[MAXHOSTNAMELEN] = 0;
 
  if(!Hosts) {
 if(ReadHosts(QHOSTSFILE) < 0)
